# -*- coding: utf-8 -*-
from collective.geo.behaviour.behaviour import Coordinates
from collective.geo.geographer.interfaces import IGeoreferenceable
from collective.geo.geographer.interfaces import IWriteGeoreferenced
from collective.jsonify.wrapper import Wrapper
from cpskin.core.interfaces import IAlbumCollection
from cpskin.core.interfaces import IBannerActivated
from cpskin.core.interfaces import IFolderViewSelectedContent
from cpskin.core.interfaces import IFolderViewWithBigImages
from cpskin.core.interfaces import ILocalBannerActivated
from cpskin.core.interfaces import IMediaActivated
from cpskin.core.interfaces import IVideoCollection
from cpskin.core.viewlets.interfaces import IViewletMenuToolsBox
from cpskin.core.viewlets.interfaces import IViewletMenuToolsFaceted
from plone import api
from plone.app.multilingual.interfaces import ITranslationManager
from plone.app.portlets.exportimport.interfaces import IPortletAssignmentExportImportHandler  # noqa
from plone.app.portlets.exportimport.portlets import PropertyPortletAssignmentExportImportHandler  # noqa
from plone.app.portlets.interfaces import IPortletTypeInterface
from plone.portlets.constants import CONTENT_TYPE_CATEGORY
from plone.portlets.constants import CONTEXT_CATEGORY
from plone.portlets.constants import GROUP_CATEGORY
from plone.portlets.constants import USER_CATEGORY
from plone.portlets.interfaces import ILocalPortletAssignable
from plone.portlets.interfaces import ILocalPortletAssignmentManager
from plone.portlets.interfaces import IPortletAssignmentMapping
from plone.portlets.interfaces import IPortletAssignmentSettings
from plone.portlets.interfaces import IPortletManager
from xml.dom import minidom
from zope.component import getUtilitiesFor
from zope.component import queryMultiAdapter
from zope.interface import Interface
from zope.interface import providedBy

import base64
import DateTime
import os


class ISerializer(Interface):
    def __call__(value, filestore, extra=None):
        """Convert to a serializable reprentation"""


class Wrapper(Wrapper):
    """ Gets the data in a format that can be used by the
        transmogrifier blueprints in collective.jsonmigrator
    """

    def __init__(self, context):
        pc = api.portal.get_tool('portal_catalog')
        self.has_tg = 'TranslationGroup' in pc.indexes()
        super(Wrapper, self).__init__(context)

    def is_multilingual(self):
        """return true if there is fr folder on top nav"""
        portal = api.portal.get()
        return 'fr' in portal.contentIds()

    def get_archetypes_fields(self):
        """ If Archetypes is used then dump schema
        """

        try:
            from Products.Archetypes.interfaces import IBaseObject
            if not IBaseObject.providedBy(self.context):
                return
        except:
            return
        if self.is_multilingual() and self.has_tg and len(ITranslationManager(self.context).get_translations()) > 1:  # noqa
            translations = ITranslationManager(self.context).get_translations()
            portal_level = len(self.portal.getPhysicalPath())
            trans = {}
            for lang, obj in translations.items():
                trans[lang] = '/'+'/'.join(
                    obj.getPhysicalPath()[portal_level:])
            self['translations'] = trans

        fields = self.context.Schema().fields()
        for field in fields:
            fieldname = unicode(field.__name__)
            type_ = field.__class__.__name__

            fieldnames = [
                'StringField', 'BooleanField', 'LinesField',
                'IntegerField', 'TextField', 'SimpleDataGridField',
                'FloatField', 'FixedPointField', 'TALESString',
                'TALESLines', 'ZPTField', 'DataGridField', 'EmailField',
                '_StringExtensionField', 'LeadimageCaptionField',
                'ExtensionStandardTagsField', 'ExtensionHiddenTagsField',
                'ExtensionIAmTagsField', 'ExtensionISearchTagsField',
                'CheckboxField'
            ]

            if type_ in fieldnames:
                try:
                    value = field.getRaw(self.context)
                except AttributeError:
                    value = self._get_at_field_value(field)

                if callable(value) is True:
                    value = value()

                if value and type_ in ['StringField', 'TextField']:
                    try:
                        value = self.decode(value)
                    except AttributeError:
                        # maybe an int?
                        value = unicode(value)
                    except Exception, e:
                        raise Exception('problems with %s: %s' % (
                            self.context.absolute_url(), str(e))
                        )
                elif value and type_ == 'DataGridField':
                    for i, row in enumerate(value):
                        for col_key in row.keys():
                            col_value = row[col_key]
                            if type(col_value) in (unicode, str):
                                value[i][col_key] = self.decode(col_value)

                try:
                    ct = field.getContentType(self.context)
                except AttributeError:
                    ct = ''
                self[unicode(fieldname)] = value
                self[unicode('_content_type_') + fieldname] = ct

            elif type_ in ['DateTimeField']:
                value = self._get_at_field_value(field)
                if value:
                    value = DateTime.DateTime.strftime(value, '%Y-%m-%d %H:%M')
                    # value = str(self._get_at_field_value(field))
                    # value = self._get_at_field_value(field).ISO8601()
                    self[unicode(fieldname)] = value

            elif type_ in [
                'ImageField',
                'FileField',
                'AttachmentField',
                'ExtensionBlobField',
                'LeadimageBlobImageField'
            ]:

                fieldname = unicode('_datafield_' + fieldname)
                value = self._get_at_field_value(field)
                value2 = value

                if not isinstance(value, str):
                    data = getattr(value, 'data')
                    if data is None:
                        continue
                    if isinstance(data, str):
                        value = base64.b64encode(value.data)
                    else:
                        value = ''
                        while data is not None:
                            value += data.data
                            data = data.next
                        value = base64.b64encode(value)

                try:
                    max_filesize = int(
                        os.environ.get('JSONIFY_MAX_FILESIZE', 20000000)
                    )
                except ValueError:
                    max_filesize = 20000000

                if value and len(value) < max_filesize:
                    size = value2.getSize()
                    try:
                        fname = field.getFilename(self.context)
                    except AttributeError:
                        fname = value2.getFilename()

                    try:
                        fname = self.decode(fname)
                    except AttributeError:
                        # maybe an int?
                        fname = unicode(fname)
                    except Exception, e:
                        raise Exception(
                            'problems with %s: %s' % (
                                self.context.absolute_url(), str(e)
                            )
                        )

                    try:
                        ctype = field.getContentType(self.context)
                    except AttributeError:
                        ctype = value2.getContentType()

                    self[fieldname] = {
                        'data': value,
                        'size': size,
                        'filename': fname or '',
                        'content_type': ctype,
                        'encoding': 'base64'
                    }

            elif type_ in ['QueryField']:
                value = field.getRaw(self.context)
                self[fieldname] = [dict(q) for q in value]

            elif type_ in ['ReferenceField', 'OrderableReferenceField']:
                pass

            elif type_ in ['ComputedField']:
                continue
            else:
                raise TypeError('Unknown field type for ArchetypesWrapper in '
                        '%s %s in %s' % (type_, fieldname, self.context.absolute_url()))  # noqa

    def get_cpskin_interfaces(self):
        from cpskin.menu.interfaces import IDirectAccess
        from cpskin.menu.interfaces import IFourthLevelNavigation
        interfaces = [
            IAlbumCollection,
            IBannerActivated,
            IFolderViewSelectedContent,
            IFolderViewWithBigImages,
            ILocalBannerActivated,
            IMediaActivated,
            IVideoCollection,
            IViewletMenuToolsBox,
            IViewletMenuToolsFaceted,
            IDirectAccess,
            IFourthLevelNavigation
        ]
        inter = []
        for interface in interfaces:
            if interface.providedBy(self.context):
                inter.append(interface.__identifier__)
        self['cpskin_interfaces'] = inter

    def get_facted_criteria(self):
        from eea.facetednavigation.criteria.handler import Criteria
        from eea.facetednavigation.criteria.interfaces import ICriteria
        from eea.facetednavigation.indexes.language.interfaces import ILanguageWidgetAdapter  # noqa
        from eea.facetednavigation.interfaces import IFacetedNavigable
        from eea.facetednavigation.settings.interfaces import IDisableSmartFacets  # noqa
        from eea.facetednavigation.settings.interfaces import IHidePloneLeftColumn  # noqa
        from eea.facetednavigation.settings.interfaces import IHidePloneRightColumn  # noqa
        from eea.facetednavigation.subtypes.interfaces import IFacetedWrapper
        from eea.facetednavigation.views.interfaces import IViewsInfo
        from eea.facetednavigation.widgets.alphabetic.interfaces import IAlphabeticWidget  # noqa
        from eea.facetednavigation.widgets.interfaces import ICriterion
        from eea.facetednavigation.widgets.interfaces import IWidget
        from eea.facetednavigation.widgets.interfaces import IWidgetsInfo
        from eea.facetednavigation.widgets.resultsfilter.interfaces import IResultsFilterWidget  # noqa

        interfaces = [
            IFacetedNavigable,
            IDisableSmartFacets,
            IHidePloneLeftColumn,
            IHidePloneRightColumn,
            ICriteria,
            ILanguageWidgetAdapter,
            IFacetedWrapper,
            IViewsInfo,
            IAlphabeticWidget,
            ICriterion,
            IWidget,
            IWidgetsInfo,
            IResultsFilterWidget,
        ]
        inter = []
        for interface in interfaces:
            if interface.providedBy(self.context):
                inter.append(interface.__identifier__)
        self['faceted_interfaces'] = inter
        if IFacetedNavigable.providedBy(self.context):
            criteria = Criteria(self.context)
            criterias = []
            for crit in criteria.criteria:
                criterias.append(crit.__dict__)
            self['faceted_criteria'] = criterias

    def get_dexterity_fields(self):
        """If dexterity is used then extract fields.
        """
        try:
            from plone.dexterity.interfaces import IDexterityContent
            if not self.providedBy(IDexterityContent, self.context):
                return
            from plone.dexterity.utils import iterSchemata
            # from plone.uuid.interfaces import IUUID
            from zope.schema import getFieldsInOrder
            from datetime import date
        except:
            return
        # get translation if thereis
        try:
            if self.is_multilingual() and self.has_tg and len(ITranslationManager(self.context).get_translations()) > 1:
                trans = ITranslationManager(self.context).get_translations()
            else:
                trans = {}
        except:
            trans = {}
        if len(trans) > 1:
            translations = ITranslationManager(self.context).get_translations()
            portal_level = len(self.portal.getPhysicalPath())
            trans = {}
            for lang, obj in translations.items():
                trans[lang] = '/'+'/'.join(
                    obj.getPhysicalPath()[portal_level:])
            self['translations'] = trans
        # get all fields for this obj
        for schemata in iterSchemata(self.context):
            for fieldname, field in getFieldsInOrder(schemata):
                try:
                    value = field.get(schemata(self.context))
                    # value = getattr(context, name).__class__.__name__
                except AttributeError:
                    continue
                if value is field.missing_value:
                    continue

                field_type = field.__class__.__name__

                if field_type in ('RichText',):
                    # XXX: content_type missing
                    try:
                        value = unicode(value.raw)
                    except:
                        value = u''

                elif field_type in (
                    'NamedImage',
                    'NamedBlobImage',
                    'NamedFile',
                    'NamedBlobFile'
                ):
                    # still to test above with NamedFile & NamedBlobFile
                    fieldname = unicode('_datafield_' + fieldname)

                    if hasattr(value, 'open'):
                        data = value.open().read()
                    else:
                        data = value.data

                    try:
                        max_filesize = int(
                            os.environ.get('JSONIFY_MAX_FILESIZE', 20000000))
                    except ValueError:
                        max_filesize = 20000000

                    if data and len(data) > max_filesize:
                        continue

                    import base64
                    ctype = value.contentType
                    size = value.getSize()
                    dvalue = {
                        'data': base64.b64encode(data),
                        'size': size,
                        'filename': value.filename or '',
                        'content_type': ctype,
                        'encoding': 'base64'
                    }
                    value = dvalue

                elif field_type == 'GeolocationField':
                    # super special plone.formwidget.geolocation case
                    self['latitude'] = getattr(value, 'latitude', 0)
                    self['longitude'] = getattr(value, 'longitude', 0)
                    continue
                elif field_type == 'ContactChoice':
                    pos = getattr(self.context, fieldname, None)
                    if pos:
                        value = unicode(pos.to_path)
                elif isinstance(value, date):
                    value = value.isoformat()

                # elif field_type in ('TextLine',):
                else:
                    BASIC_TYPES = (
                        unicode, int, long, float, bool, type(None),
                        list, tuple, dict
                    )
                    if type(value) in BASIC_TYPES:
                        pass
                    else:
                        # E.g. DateTime or datetime are nicely representated
                        value = unicode(value)

                self[unicode(fieldname)] = value

    def get_portlets(self):
        """If there is portlets then extract its.
        """

        self.doc = minidom.Document()
        self.portlet_schemata = dict([(iface, name,) for name, iface in getUtilitiesFor(IPortletTypeInterface)])  # noqa
        self.portlet_managers = list(getUtilitiesFor(IPortletManager))
        if ILocalPortletAssignable.providedBy(self.context):
            data = None

            root = self.doc.createElement('portlets')

            for elem in self.exportAssignments(self.context):
                root.appendChild(elem)
            for elem in self.exportBlacklists(self.context):
                root.appendChild(elem)
            if root.hasChildNodes():
                self.doc.appendChild(root)
                data = self.doc.toprettyxml(indent='  ', encoding='utf-8')
                self.doc.unlink()

            if data:
                self['portlets'] = data

    def exportAssignments(self, obj):
        assignments = []
        for manager_name, manager in self.portlet_managers:
            mapping = queryMultiAdapter(
                (obj, manager),
                IPortletAssignmentMapping)
            if mapping is None:
                continue

            mapping = mapping.__of__(obj)
            if len(mapping.items()) > 0:
                for name, assignment in mapping.items():
                    type_ = None
                    for schema in providedBy(assignment).flattened():
                        type_ = self.portlet_schemata.get(schema, None)
                        if type_ is not None:
                            break

                    if type_ is not None:
                        child = self.doc.createElement('assignment')
                        child.setAttribute('manager', manager_name)
                        child.setAttribute('category', CONTEXT_CATEGORY)
                        child.setAttribute('key', '/'.join(
                            obj.getPhysicalPath()))
                        child.setAttribute('type', type_)
                        child.setAttribute('name', name)
                        settings = IPortletAssignmentSettings(assignment)
                        if not settings.get('visible', True):
                            child.setAttribute('visible', 'False')

                        assignment = assignment.__of__(mapping)
                        # use existing adapter for exporting a portlet assignment  # noqa
                        handler = IPortletAssignmentExportImportHandler(assignment)  # noqa
                        handler.export_assignment(schema, self.doc, child)

                        assignments.append(child)

        return assignments

    def exportBlacklists(self, obj):
        assignments = []
        for manager_name, manager in self.portlet_managers:
            assignable = queryMultiAdapter(
                (obj, manager),
                ILocalPortletAssignmentManager)
            if assignable is None:
                continue
            for category in (
                USER_CATEGORY,
                GROUP_CATEGORY,
                CONTENT_TYPE_CATEGORY,
                CONTEXT_CATEGORY
            ):
                child = self.doc.createElement('blacklist')
                child.setAttribute('manager', manager_name)
                child.setAttribute('category', category)

                status = assignable.getBlacklistStatus(category)
                if status is True:
                    child.setAttribute('status', u'block')
                elif status is False:
                    child.setAttribute('status', u'show')
                else:
                    child.setAttribute('status', u'acquire')

                assignments.append(child)

        return assignments

    def get_georeference(self):
        if IGeoreferenceable.providedBy(self.context):
            try:
                IWriteGeoreferenced(self.context)
                coord = Coordinates(self.context).coordinates
                self['coordinates'] = coord
            except:
                pass

    def get_comments(self):
        if self.context.portal_type == 'Discussion Item':
            self['author_email'] = getattr(self.context, 'author_email', None)
            self['author_name'] = getattr(self.context, 'author_name', None)
            self['author_username'] = getattr(self.context, 'author_username', None)

    def get_newsletter(self):
        pt = self.context.portal_type
        if pt == 'NewsletterTheme':
            self['testEmail'] = getattr(self.context, 'testEmail', None)
            self['authorEmail'] = getattr(self.context, 'authorEmail', None)
            self['replyto'] = getattr(self.context, 'replyto', None)
            self['activationMailSubject'] = getattr(self.context, 'activationMailSubject', None)
            self['activationMailTemplate'] = getattr(self.context, 'activationMailTemplate', None)
            self['newsletterHeader'] = getattr(self.context, 'newsletterHeader', None)
            self['newsletterFooter'] = getattr(self.context, 'newsletterFooter', None)
            self['newsletterStyle'] = getattr(self.context, 'newsletterStyle', None)
            self['notify'] = getattr(self.context, 'notify', False)
            self['renderTemplate'] = getattr(self.context, 'renderTemplate', None)
            self['extraRecipients'] = getattr(self.context, 'extraRecipients', None)
            self['subscriber_folder_id'] = getattr(self.context, 'subscriber_folder_id', None)
            self['alternative_portal_url'] = getattr(self.context, 'alternative_portal_url', None)
        if pt == 'Newsletter':
            self['setFormat'] = getattr(self.context, 'setFormat', None)
            self['dateEmitted'] = getattr(self.context, 'dateEmitted', None)



# -*- coding: utf-8 -*-
"""
cpskin.core
-----------

Created by mpeeters
:license: GPL, see LICENCE.txt for more details.
"""

from collective.contact.facetednav.browser.view import PreviewItem
from cpskin.core import utils

import six


class FacetedPreviewItem(PreviewItem):

    @property
    def phones(self):
        phones = getattr(self.context, 'phone', [])
        if isinstance(phones, six.string_types):
            phones = [phones]
        return [utils.format_phone(v) for v in phones]

    @property
    def cell_phones(self):
        cell_phones = getattr(self.context, 'cell_phone', [])
        if isinstance(cell_phones, six.string_types):
            cell_phones = [cell_phones]
        return [utils.format_phone(v) for v in cell_phones]

    @property
    def fax(self):
        fax = getattr(self.context, 'fax', None)
        if fax:
            return utils.format_phone(fax)



# -*- coding: utf-8 -*-

from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from plone import api
from plone.app.users.browser.register import RegistrationForm
from zope import schema
from zope.formlib import form
from zope.formlib.interfaces import WidgetInputError
from zope.interface import Interface

from cpskin.locales import CPSkinMessageFactory as _


class ICustomRegisterSchema(Interface):

    legal_conditions = schema.Bool(
        title=_(u'I Accept Legal terms and conditions'),
        description=_(u''),
        required=True,
    )


class CustomRegistrationForm(RegistrationForm):
    correct_captcha = None
    template = ViewPageTemplateFile('templates/register_form.pt')

    @property
    def form_fields(self):
        base_fields = super(CustomRegistrationForm, self).form_fields
        custom_fields = form.Fields(ICustomRegisterSchema)
        portal_url = api.portal.get().absolute_url()
        gpdr_url = '/'.join([portal_url, 'gdpr-view'])
        msgid = _(
            u'see_legal_conditions',
            default=u'See <a href="${legal_conditions_url}">legal terms and conditions</a>.',  #NOQA
            mapping={u'legal_conditions_url': gpdr_url},
        )
        legal_description = self.context.translate(msgid)
        custom_fields['legal_conditions'].field.description = legal_description
        return base_fields + custom_fields

    def validate_registration(self, action, data):
        errors = super(CustomRegistrationForm, self).validate_registration(
            action,
            data,
        )
        if not data.get('legal_conditions'):
            err_str = _(u'You need to accept our legal terms and conditions.')
            error = WidgetInputError(
                'legal_conditions',
                _(u'I Accept Legal terms and conditions'),
                err_str,
            )
            errors.append(error)
            self.widgets['legal_conditions'].error = err_str
        correct_captcha = self.context.restrictedTraverse('@@captcha').verify()
        if not correct_captcha:
            error = WidgetInputError(
                'captcha',
                'Captcha',
                _(u'Incorrect captcha')
            )
            errors.append(error)
        self.correct_captcha = correct_captcha
        return errors



# -*- coding: utf-8 -*-
from collective.geo.behaviour.interfaces import ICoordinates
from cpskin.core.testing import CPSKIN_CORE_INTEGRATION_TESTING
from cpskin.core.utils import add_behavior
from plone import api
from plone.app.testing import applyProfile
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from zope.event import notify
from zope.lifecycleevent import ObjectModifiedEvent

import unittest


class TestEvents(unittest.TestCase):

    layer = CPSKIN_CORE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

    def test_modified_orga_events(self):
        # add some contacts
        applyProfile(self.portal, 'collective.contact.core:default')
        add_behavior('organization', ICoordinates.__identifier__)
        directory = api.content.create(
            container=self.portal, type='directory', id='directory')
        orga = api.content.create(
            container=directory, type='organization', id='orga')
        orga.use_parent_address = False
        orga.street = u'Rue Léon Morel'
        orga.number = u'1'
        # notify(ObjectAddedEvent(orga))
        coord = ICoordinates(orga).coordinates
        self.assertFalse(coord.startswith('POINT '))
        orga.zip_code = u'5032'
        orga.city = u'Isnes'
        notify(ObjectModifiedEvent(orga))
        coord = ICoordinates(orga).coordinates
        self.assertTrue(coord.startswith('POINT '))



# -*- coding: utf-8 -*-
from Acquisition import aq_base
from collective import dexteritytextindexer
from collective.contact.core.content.organization import IOrganization
from collective.taxonomy.interfaces import ITaxonomy
from OFS.interfaces import IItem
from plone.app.contenttypes.indexers import _unicode_save_string_concat
from plone.app.contenttypes.indexers import SearchableText_document
from plone.app.contenttypes.indexers import SearchableText_file
from plone.app.contenttypes.indexers import SearchableText_link
from plone.app.contenttypes.interfaces import IDocument
from plone.app.contenttypes.interfaces import IFile
from plone.app.contenttypes.interfaces import ILink
from plone.indexer.interfaces import IIndexer
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import safe_unicode
from Products.ZCatalog.interfaces import IZCatalog
from plone import api
from zope.component import adapts
from zope.component import getUtility
from zope.component.interfaces import ComponentLookupError
from zope.interface import implements

import logging
import six


try:
    from plone.dexterity.utils import safe_utf8
except ImportError:
    # Compatibility with Plone 4.3.2 / plone.dexterity 2.1.3
    from cpskin.core.utils import safe_utf8

logger = logging.getLogger('cpskin.core.indexer')


class DocumentExtender(object):
    adapts(IDocument)
    implements(dexteritytextindexer.IDynamicTextIndexExtender)

    def __init__(self, context):
        self.context = context

    def __call__(self):
        return SearchableText_document(self.context)()


class LinkExtender(object):
    adapts(ILink)
    implements(dexteritytextindexer.IDynamicTextIndexExtender)

    def __init__(self, context):
        self.context = context

    def __call__(self):
        return SearchableText_link(self.context)()


class FileExtender(object):
    adapts(IFile)
    implements(dexteritytextindexer.IDynamicTextIndexExtender)

    def __init__(self, context):
        self.context = context

    def __call__(self):
        return SearchableText_file(self.context)()


class OrganizationExtender(object):
    adapts(IOrganization)
    implements(dexteritytextindexer.IDynamicTextIndexExtender)

    def __init__(self, context):
        self.context = context

    def __call__(self):
        rich_text_ids = ["activity", "informations_complementaires"]
        text_items = []
        for rich_text_id in rich_text_ids:
            rich_text = getattr(self.context, rich_text_id, None)
            if rich_text is None:
                continue
            transforms = getToolByName(self.context, "portal_transforms")
            raw = safe_unicode(rich_text.raw)
            if six.PY2:
                raw = raw.encode("utf-8", "replace")
            text_items.append(
                safe_unicode(
                    transforms.convertTo("text/plain", raw, mimetype=rich_text.mimeType)
                    .getData()
                    .strip()
                )
            )
        text = u" ".join(text_items)

        taxonomy_ids = api.portal.get_registry_record(
            'cpskin.core.interfaces.ICPSkinSettings.indexed_taxonomies')  # noqa
        taxonomy_ids = taxonomy_ids.split("\n")
        taxo_items = []
        lang = self.context.language
        for taxonomy_id in taxonomy_ids:
            field_name = taxonomy_id.replace("_", "")
            keys = getattr(self.context, "taxonomy_{0}".format(field_name), None)
            if not keys:
                continue
            try:
                taxonomy = getUtility(
                    ITaxonomy, name="collective.taxonomy.{0}".format(taxonomy_id)
                )
            except ComponentLookupError:
                taxonomy = getUtility(
                    ITaxonomy, name="collective.taxonomy.{0}".format(field_name)
                )
            for key in keys:
                try:
                    taxo_items.append(
                        safe_unicode(
                            taxonomy.translate(
                                key, context=self.context, target_language=lang
                            )
                        )
                    )
                except KeyError:
                    pass
        taxo_text = u" ".join(taxo_items)

        return _unicode_save_string_concat(
            u" ".join(
                (
                    safe_unicode(self.context.id),
                    safe_unicode(self.context.title) or u"",
                    safe_unicode(self.context.description) or u"",
                    text,
                    taxo_text,
                )
            )
        )


class BaseTagIndexer(object):
    """Index the specified tag
    """
    implements(IIndexer)
    adapts(IItem, IZCatalog)

    def __init__(self, context, catalog):
        self.context = context
        self.catalog = catalog

    def _getFieldContent(self, field):
        tags = getattr(aq_base(self.context), field, None)
        if not tags:
            raise AttributeError
        if field not in self.context.contentIds():
            encodedTags = tuple(safe_utf8(s) for s in tags)
            return encodedTags


class StandardTagIndexer(BaseTagIndexer):

    def __call__(self):
        return self._getFieldContent('standardTags')


class IAmTagIndexer(BaseTagIndexer):

    def __call__(self):
        return self._getFieldContent('iamTags')


class ISearchTagIndexer(BaseTagIndexer):

    def __call__(self):
        return self._getFieldContent('isearchTags')


class HiddenTagIndexer(BaseTagIndexer):

    def __call__(self):
        return self._getFieldContent('hiddenTags')



# -*- coding: utf-8 -*-

from Products.Five import BrowserView
from collective.taxonomy import PATH_SEPARATOR
from collective.taxonomy.interfaces import ITaxonomy
from plone import api
from plone.dexterity.interfaces import IDexterityFTI
from zope.component import getSiteManager
from zope.component import getUtility
from zope.component import queryUtility
from zope.schema import getFields
from zope.schema.interfaces import IVocabularyFactory


class CommonView(BrowserView):

    def translate_taxonomy(self, msgid, domain=''):
        sm = getSiteManager()
        utility = sm.queryUtility(ITaxonomy, name=domain)
        target_language = str(utility.getCurrentLanguage(self.request))
        if msgid not in utility.inverted_data[target_language]:
            return ''
        path = utility.inverted_data[target_language][msgid]
        sub_category = path[1:].split(PATH_SEPARATOR)[-1]
        return sub_category

    def see_categories(self, container):
        result = True
        taxonomy_field = getattr(container, 'taxonomy_category', '')
        if not taxonomy_field:
            result = False
        return result

    def get_field_taxonomy(self, obj, field, taxonomy_field, limit=None):
        vocabulary_name = field.value_type.vocabularyName
        factory = getUtility(IVocabularyFactory, vocabulary_name)
        vocabulary = factory(api.portal.get())
        tokens = getattr(obj, taxonomy_field, '')
        if not tokens:
            return ''
        categories = []
        for token in tokens:
            cat = vocabulary.inv_data.get(token)
            if not cat:
                return ''
            # format is "/taxlevel1/taxlevel2/..." and we want the last term
            last_term = cat.split(PATH_SEPARATOR)[-1]
            categories.append(last_term)
        if limit is None:
            categories.sort()
        else:
            categories = categories[:limit]
        return ', '.join(categories)

    def get_behavior_taxonomy(self, obj, short_name, limit=None):
        categories = []
        if not getattr(obj, 'taxonomy_' + short_name, None):
            return ''
        for taxon in getattr(obj, 'taxonomy_' + short_name):
            categories.append(self.translate_taxonomy(
                taxon,
                domain='collective.taxonomy.' + short_name
            ))
        if limit is None:
            categories.sort()
        else:
            categories = categories[:limit]
        return ', '.join(categories)

    def get_categories(self, container, obj, limit=None):
        portal_type = obj.portal_type
        schema = getUtility(IDexterityFTI, name=portal_type).lookupSchema()
        fields = getFields(schema)
        taxonomy_field = getattr(container, 'taxonomy_category', '')
        if not taxonomy_field:
            return
        if taxonomy_field in fields.keys():
            field = fields[taxonomy_field]
            return self.get_field_taxonomy(obj, field, taxonomy_field, limit)

        long_name = 'collective.taxonomy.{0}'.format(
            taxonomy_field.replace('taxonomy_', '').replace('_', ''))
        utility = queryUtility(ITaxonomy, long_name)
        if utility is not None:
            short_name = utility.getShortName()
            return self.get_behavior_taxonomy(obj, short_name, limit)

        return ''



# -*- coding: utf-8 -*-
from zope.interface import Interface


class IViewletMenuToolsFaceted(Interface):
    """
    marker interface used to viewlets menutools
    """


class IViewletMenuToolsBox(Interface):
    """
    marker interface used to viewlets menutools
    """



# -*- coding: utf-8 -*-
from collective.dexteritytextindexer.behavior import IDexterityTextIndexer
from collective.taxonomy.exportimport import TaxonomyImportExportAdapter
from collective.taxonomy.factory import registerTaxonomy
from cpskin.core.behaviors.directory import ICpskinDirectoryViewSettings
from cpskin.core.behaviors.eventview import ICpskinEventViewSettings
from cpskin.core.behaviors.indexview import ICpskinIndexViewSettings
from cpskin.core.behaviors.metadata import IAdditionalSearchableText
from cpskin.core.behaviors.organization import IOrganizationImages
from cpskin.core.utils import add_behavior
from cpskin.core.utils import set_exclude_from_nav
from cpskin.locales import CPSkinMessageFactory as _
from plone import api
from plone.app.versioningbehavior.behaviors import IVersionable
from plone.app.viewletmanager.interfaces import IViewletSettingsStorage
from plone.app.viewletmanager.manager import ManageViewlets
from plone.registry import field
from plone.registry import Record
from plone.registry.interfaces import IRegistry
from Products.CMFDefault.utils import bodyfinder
from zope.component import getUtility
from zope.component import queryMultiAdapter
from zope.interface import noLongerProvides

import logging
import os


logger = logging.getLogger('cpskin.core')


def installCore(context):
    if context.readDataFile('cpskin.core-default.txt') is None:
        return

    logger.info('Installing')
    portal = api.portal.get()

    # Rename events and news
    ChangeCollectionsIds(portal)

    # Add the MaildropHost if possible
    addMaildropHost(portal)

    # Add catalog indexes
    addCatalogIndexes(portal)

    # Create default banner image
    addImageFromFile(portal, 'banner.jpg')

    # Create default visuel image
    addImageFromFile(portal, 'visuel.png')

    # Create default logo
    addImageFromFile(portal, 'cpskinlogo.png')

    add_behavior('Collection', ICpskinIndexViewSettings.__identifier__)
    add_behavior('Event', ICpskinEventViewSettings.__identifier__)
    add_behavior('organization', IOrganizationImages.__identifier__)
    add_behavior('organization', IAdditionalSearchableText.__identifier__)
    add_behavior('organization', IDexterityTextIndexer.__identifier__)
    add_behavior('organization', IVersionable.__identifier__)
    add_behavior('person', IVersionable.__identifier__)
    add_behavior('position', IVersionable.__identifier__)
    add_behavior('directory', ICpskinDirectoryViewSettings.__identifier__)
    add_behavior('directory', IVersionable.__identifier__)

    # Create footer static page
    footer_name = 'footer-static'
    if not portal.hasObject(footer_name):
        footer = api.content.create(type='Document',
                                    title=footer_name,
                                    container=portal)
        footer.setTitle(footer_name)

    configCollectiveQucikupload()

    addLoadPageMenuToRegistry()
    addAutoPlaySliderToRegistry()
    addSliderTimerToRegistry()
    addCityNameToRegistry()
    addSubMenuPersistenceToRegistry()
    # addSliderTypeToRegistry()
    set_googleapi_key()
    upgrade_front_page()
    order_portaltop_viewlets()
    add_other_xhtml_valid_tags()
    add_default_taxonomies()


def add_other_xhtml_valid_tags():
    portal_transforms = api.portal.get_tool('portal_transforms')
    safe_html = getattr(portal_transforms, 'safe_html')
    key = 'valid_tags'
    valids = safe_html.get_parameter_value('valid_tags')
    other_tags = ['figcaption']
    kwargs = {}
    kwargs[key + '_key'] = valids.keys()
    kwargs[key + '_value'] = [str(s) for s in valids.values()]
    for tag in other_tags:
        if tag not in valids.keys():
            kwargs[key + '_key'].append(tag)
            kwargs[key + '_value'].append('1')
            logger.info('Add {0} to valid html tags'.format(tag))
    safe_html.set_parameters(**kwargs)
    safe_html._p_changed = True
    safe_html.reload()


def order_portaltop_viewlets():
    storage = getUtility(IViewletSettingsStorage)
    manager_name = 'plone.portaltop'
    portal = api.portal.get()
    skinname = portal.getCurrentSkinName()
    view = ManageViewlets(portal, portal.REQUEST)
    order = view._getOrder(manager_name)
    order.insert(1, 'cpskin.banner')
    storage.setOrder(manager_name, skinname, order)


def uninstallCore(context):
    if context.readDataFile('cpskin.core-uninstall.txt') is None:
        return

    logger.info('Uninstalling')
    portal = api.portal.get()

    # Remove banner image
    if portal.hasObject('banner.jpg'):
        api.content.delete(obj=portal['banner.jpg'])

    # Remove visuel image
    if portal.hasObject('visuel.png'):
        api.content.delete(obj=portal['visuel.png'])

    # Remove default logo
    if portal.hasObject('cpskinlogo.png'):
        api.content.delete(obj=portal['cpskinlogo.png'])

    # Remove footer static
    if portal.hasObject('footer-static'):
        api.content.delete(obj=portal['footer-static'])

    unregisterProvidesInterfaces(portal)


def unregisterProvidesInterfaces(portal):
    from cpskin.core.interfaces import (
        IBannerActivated,
        IMediaActivated,
        IAlbumCollection,
        IVideoCollection
    )

    interfaces = [IBannerActivated,
                  IMediaActivated,
                  IAlbumCollection,
                  IVideoCollection]
    for interface in interfaces:
        catalog = api.portal.get_tool('portal_catalog')
        brains = catalog({
            'object_provides': interface.__identifier__,
        })
        for brain in brains:
            obj = brain.getObject()
            noLongerProvides(obj, interface)
            obj.reindexObject()


def setPageText(portal, page, viewName):
    """
    Sets text of a Plone document if it exists and reindex the document
    The text is coming from a browser view template <body> tag
    """
    if page is None:
        return
    request = getattr(portal, 'REQUEST', None)
    if request is not None:
        view = queryMultiAdapter((portal, request), name=viewName)
        if view is not None:
            text = bodyfinder(view.index()).strip()
            try:
                page.setText(text, mimetype='text/html')
            except:
                from plone.app.textfield.value import RichTextValue
                rtv = RichTextValue(text)
                page.text = rtv
            page.reindexObject()


def addMaildropHost(self):
    """
     Add a MaildropHost if Products.MaildropHost is available...
     If MaildropHost exist, PloneGazette will use it to send mails.
     This will avoid duplicate emails send as reported by
    """
    portal = api.portal.get_tool('portal_url').getPortalObject()
    if not getattr(portal, 'MaildropHost', None):
        try:
            portal.manage_addProduct['MaildropHost'].manage_addMaildropHost(
                'MaildropHost', title='MaildropHost')
        except AttributeError:
            # if MaildropHost is not available, we pass...
            pass


def addCatalogIndexes(portal):
    """
    Method to add our wanted indexes to the portal_catalog.
    We couldn't do it in the profile directly, see :
        http://maurits.vanrees.org/weblog/archive/2009/12/catalog
    """
    catalog = api.portal.get_tool('portal_catalog')
    indexes = catalog.indexes()
    wanted = (('standardTags', 'KeywordIndex'),
              ('iamTags', 'KeywordIndex'),
              ('isearchTags', 'KeywordIndex'),
              ('hiddenTags', 'KeywordIndex'))
    indexables = []
    for name, meta_type in wanted:
        if name not in indexes:
            catalog.addIndex(name, meta_type)
            indexables.append(name)
            logger.info('Added {0} for field {1}.'.format(meta_type, name))
    if len(indexables) > 0:
        logger.info('Indexing new indexes {0}.'.format(', '.join(indexables)))
        catalog.manage_reindexIndex(ids=indexables)


def ChangeCollectionsIds(portal):
    # required : topic object need a _p_jar for rename
    import transaction
    transaction.savepoint()
    if portal.hasObject('news'):
        news = portal['news']
        if news.hasObject('aggregator'):
            set_exclude_from_nav(news['aggregator'])
            api.content.rename(obj=news['aggregator'], new_id='index')
        set_exclude_from_nav(news)
        api.content.rename(obj=news, new_id='actualites')
    if portal.hasObject('events'):
        events = portal['events']
        if events.hasObject('aggregator'):
            set_exclude_from_nav(events['aggregator'])
            api.content.rename(obj=events['aggregator'], new_id='index')
        set_exclude_from_nav(events)
        api.content.rename(obj=events, new_id='evenements')


def addImageFromFile(portal, fileName):
    dataPath = os.path.join(os.path.dirname(__file__), 'data')
    filePath = os.path.join(dataPath, fileName)
    if not portal.hasObject(fileName):
        # with deterity image
        from plone.namedfile.file import NamedBlobImage
        namedblobimage = NamedBlobImage(
            data=open(filePath, 'r').read(),
            filename=unicode(fileName)
        )
        image = api.content.create(type='Image',
                                   title=fileName,
                                   image=namedblobimage,
                                   container=portal,
                                   language='fr')
        image.setTitle(fileName)
        image.reindexObject()


def configCollectiveQucikupload():
    ptool = api.portal.get_tool('portal_properties')
    qu_props = ptool.get('quickupload_properties')
    if not qu_props.hasProperty('show_upload_action'):
        qu_props._setProperty('show_upload_action', True, 'boolean')
    else:
        qu_props.show_upload_action = True
    if not qu_props.hasProperty('sim_upload_limit'):
        qu_props._setProperty('sim_upload_limit', 1, 'int')
    else:
        qu_props.sim_upload_limit = 1
    if not qu_props.hasProperty('size_limit'):
        qu_props._setProperty('size_limit', 30000, 'int')
    else:
        qu_props.sim_upload_limit = 30000


def addLoadPageMenuToRegistry():
    registry = getUtility(IRegistry)
    records = registry.records
    if 'cpskin.core.interfaces.ICPSkinSettings.load_page_menu' in records:
        return

    logger.info(
        'Adding cpskin.core.interfaces.ICPSkinSettings.load_page_menu to registry')  # noqa
    record = Record(field.Bool(title=_(u'Load page menu'),
                               description=_(
                                   u'Is level 1 menu load page at click?'),
                               required=False,
                               default=False),
                    value=False)
    records['cpskin.core.interfaces.ICPSkinSettings.load_page_menu'] = record


def addSubMenuPersistenceToRegistry():
    registry = getUtility(IRegistry)
    records = registry.records
    if 'cpskin.core.interfaces.ICPSkinSettings.sub_menu_persistence' in records:  # noqa
        return

    logger.info(
        'Adding cpskin.core.interfaces.ICPSkinSettings.sub_menu_persistence to registry')  # noqa
    record = Record(field.Bool(title=_(u'Sub menu persistence'),
                               description=_(u'Is level 2 menu persist?'),
                               required=False,
                               default=True),
                    value=True)
    records['cpskin.core.interfaces.ICPSkinSettings.sub_menu_persistence'] = record  # noqa


def addAutoPlaySliderToRegistry():
    registry = getUtility(IRegistry)
    records = registry.records
    if 'cpskin.core.interfaces.ICPSkinSettings.auto_play_slider' in records:
        return

    logger.info(
        'Adding cpskin.core.interfaces.ICPSkinSettings.auto_play_slider to registry')  # noqa
    record = Record(
        field.Bool(
            title=_(u'Auto play slider'),
            description=_(u'Is the front page slider automatically play?'),
            required=False,
            default=True),
        value=True)
    records['cpskin.core.interfaces.ICPSkinSettings.auto_play_slider'] = record


def addSliderTimerToRegistry():
    registry = getUtility(IRegistry)
    records = registry.records
    if 'cpskin.core.interfaces.ICPSkinSettings.slider_timer' in records:
        return

    logger.info(
        'Adding cpskin.core.interfaces.ICPSkinSettings.slider_timer to registry')  # noqa
    record = Record(
        field.Int(
            title=_(u'Slider timer'),
            description=_(u'Number of seconds between each transition.'),
            required=False,
            default=5000),
        value=5000)
    records['cpskin.core.interfaces.ICPSkinSettings.slider_timer'] = record


def addCityNameToRegistry():
    registry = getUtility(IRegistry)
    records = registry.records
    if 'cpskin.core.interfaces.ICPSkinSettings.city_name' in records:
        return

    logger.info(
        'Adding cpskin.core.interfaces.ICPSkinSettings.city_name to registry')
    portal = api.portal.get()
    site_id = portal.getId()
    city_name = unicode(site_id.capitalize())
    record = Record(
        field.TextLine(
            title=_(u'City name'),
            description=_(u'City name variable used in some template.'),
            required=True,
            default=u'City name'),
        value=city_name)
    records['cpskin.core.interfaces.ICPSkinSettings.city_name'] = record


def addSliderTypeToRegistry():
    registry = getUtility(IRegistry)
    records = registry.records
    if 'cpskin.core.interfaces.ICPSkinSettings.slider_type' in records:
        return

    logger.info(
        'Adding cpskin.core.interfaces.ICPSkinSettings.slider_type to registry')  # noqa
    record = Record(
        field.TextLine(
            title=_(u'Slider type'),
            description=_(u'Choose an horizontal or vertical slider.'),
            required=True,
            default=u'slider_view'),
        value=u'slider_view')
    records['cpskin.core.interfaces.ICPSkinSettings.slider_type'] = record


def addPortletsInRightActionsToRegistry():
    registry = getUtility(IRegistry)
    records = registry.records

    if 'cpskin.core.interfaces.ICPSkinSettings.show_portlets_in_right_actions_panel' in records:  # noqa
        return

    logger.info(
        'Adding cpskin.core.interfaces.ICPSkinSettings.show_portlets_in_right_actions_panel to registry')  # noqa
    record = Record(field.Bool(title=_(u'Show (right) portlets in right actions panel'),
                               description=_(u'Show (right) portlets (if any) in right actions panel after related contents.'),  # noqa
                               required=False,
                               default=False),
                    value=False)
    records['cpskin.core.interfaces.ICPSkinSettings.show_portlets_in_right_actions_panel'] = record  # noqa


def addTopMenuLeadImageToRegistry():
    registry = getUtility(IRegistry)
    records = registry.records

    if 'cpskin.core.interfaces.ICPSkinSettings.show_leadimage_in_action_menu' in records:  # noqa
        return

    logger.info(
        'Adding cpskin.core.interfaces.ICPSkinSettings.show_leadimage_in_action_menu to registry')  # noqa
    record = Record(field.Bool(title=_(u'Show leadimage in action menu'),
                               description=_(u'Show leadimage (if any) in the top action menu with content selected in the field before.'),  # noqa
                               required=False,
                               default=False),
                    value=False)
    records['cpskin.core.interfaces.ICPSkinSettings.show_leadimage_in_action_menu'] = record  # noqa


def addPersonContactCoreFallbackToRegistry():
    registry = getUtility(IRegistry)
    records = registry.records

    if 'cpskin.core.interfaces.ICPSkinSettings.person_contact_core_fallback' in records:  # noqa
        return

    logger.info(
        'Adding cpskin.core.interfaces.ICPSkinSettings.person_contact_core_fallback to registry')  # noqa
    record = Record(field.Bool(title=_(u'Contact properties fallback?'),
                               description=_(u'Do you want than contact properties fallback? Sample :If no tel on a person, so we get phone thanks to person\'f function.'),  # noqa
                               required=False,
                               default=False),
                    value=False)
    records['cpskin.core.interfaces.ICPSkinSettings.person_contact_core_fallback'] = record  # noqa


def addTopMenuContentsToRegistry():
    registry = getUtility(IRegistry)
    records = registry.records
    if 'cpskin.core.interfaces.ICPSkinSettings.contents_in_action_menu' in records:  # noqa
        return

    logger.info(
        'Adding cpskin.core.interfaces.ICPSkinSettings.contents_in_action_menu to registry')  # noqa
    record = Record(field.Tuple(title=_(u'Content to show in special action menu (top)'),
                                description=_(u'Please select which contents should be taken to this menu.'),  # noqa
                                value_type=field.TextLine(title=u"Value"),
                                required=False),
                    )
    records['cpskin.core.interfaces.ICPSkinSettings.contents_in_action_menu'] = record  # noqa


def addShowSloganToRegistry():
    registry = getUtility(IRegistry)
    records = registry.records

    if 'cpskin.core.interfaces.ICPSkinSettings.show_slogan' in records:
        return

    logger.info(
        'Adding cpskin.core.interfaces.ICPSkinSettings.show_slogan to registry')  # noqa
    record = Record(field.Bool(title=_(u'Show slogan with banner'),
                               description=_(u'Show banner title and description as homepage slogan.'),  # noqa
                               required=False,
                               default=False),
                    value=False)
    records['cpskin.core.interfaces.ICPSkinSettings.show_slogan'] = record  # noqa


def addMediaViewletAlbumsToRegistry(value):
    registry = getUtility(IRegistry)
    records = registry.records

    if 'cpskin.core.interfaces.ICPSkinSettings.media_viewlet_visible_albums' in records:
        return

    logger.info(
        'Adding cpskin.core.interfaces.ICPSkinSettings.media_viewlet_visible_albums to registry')  # noqa
    record = Record(field.Int(title=_(u'Viewlet media : Visible albums'),
                               description=_(u'Number of visible albums on media viewlet.'),  # noqa
                               required=False,
                               default=1),
                    value=value)
    records['cpskin.core.interfaces.ICPSkinSettings.media_viewlet_visible_albums'] = record  # noqa


def addMediaViewletVideosToRegistry(value):
    registry = getUtility(IRegistry)
    records = registry.records

    if 'cpskin.core.interfaces.ICPSkinSettings.media_viewlet_visible_videos' in records:
        return

    logger.info(
        'Adding cpskin.core.interfaces.ICPSkinSettings.media_viewlet_visible_videos to registry')  # noqa
    record = Record(field.Bool(title=_(u'Viewlet media : Visible videos'),
                               description=_(u'Number of visible videos on media viewlet.'),  # noqa
                               required=False,
                               default=1),
                    value=value)
    records['cpskin.core.interfaces.ICPSkinSettings.media_viewlet_visible_videos'] = record  # noqa


def addMediaViewletOptionsToRegistry(upgrade=False):
    registry = getUtility(IRegistry)
    records = registry.records

    value = 1
    if 'cpskin.core.default_visible_albums' in records:
        logger.info('Using existing value for visible_albums')
        value = records['cpskin.core.default_visible_albums'].value
        del records['cpskin.core.default_visible_albums']
    addMediaViewletAlbumsToRegistry(value)

    value = 1
    if upgrade:
        value = 2  # old default value was fixed in Python code
    addMediaViewletVideosToRegistry(value)


def addDescriptionOnThemesOptionToRegistry():
    registry = getUtility(IRegistry)
    records = registry.records

    if 'cpskin.core.interfaces.ICPSkinSettings.show_description_on_themes' in records:  # noqa
        return

    logger.info(
        'Adding cpskin.core.interfaces.ICPSkinSettings.show_description_on_themes to registry')  # noqa
    record = Record(field.Bool(title=_(u'Show description on themes'),
                               description=_(u'Add content description after every portal tab menu title.'),  # noqa
                               required=False,
                               default=False),
                    value=False)
    records['cpskin.core.interfaces.ICPSkinSettings.show_description_on_themes'] = record  # noqa


def addCollapseMinisiteMenuToRegistry():
    registry = getUtility(IRegistry)
    records = registry.records

    if 'cpskin.core.interfaces.ICPSkinSettings.collapse_minisite_menu' in records:  # noqa
        return

    logger.info(
        'Adding cpskin.core.interfaces.ICPSkinSettings.collapse_minisite_menu to registry')  # noqa
    record = Record(field.Bool(title=_(u'Collapse menu on minisites'),
                               description=_(u'Automatically collapse portal main menu on minisites.'),  # noqa
                               required=False,
                               default=False),
                    value=False)
    records['cpskin.core.interfaces.ICPSkinSettings.collapse_minisite_menu'] = record  # noqa


def addFooterSitemapToRegistry():
    registry = getUtility(IRegistry)
    records = registry.records

    if 'cpskin.core.interfaces.ICPSkinSettings.show_footer_sitemap' in records:  # noqa
        return

    logger.info(
        'Adding cpskin.core.interfaces.ICPSkinSettings.show_footer_sitemap to registry')  # noqa
    record = Record(field.Bool(title=_(u'Show footer sitemap'),
                               description=_(u'Automatically generate sitemap footer.'),
                               required=False,
                               default=True),
                    value=True)
    records['cpskin.core.interfaces.ICPSkinSettings.show_footer_sitemap'] = record  # noqa


def addContentClassesToRegistry():
    registry = getUtility(IRegistry)
    records = registry.records

    if 'cpskin.core.interfaces.ICPSkinSettings.header_class' in records:  # noqa
        return

    logger.info(
        'Adding cpskin.core.interfaces.ICPSkinSettings.header_class to registry')  # noqa
    record = Record(field.Choice(title=_(u'Header related class'),
                                 description=_(u'CSS header class that will be applied to body.'),  # noqa
                                 required=False,
                                 values=[
                                     u'header-1',
                                     u'header-2',
                                     u'header-3',
                                     u'header-4',
                                 ],
                                 default=None),
                    value=None)
    records['cpskin.core.interfaces.ICPSkinSettings.header_class'] = record

    logger.info(
        'Adding cpskin.core.interfaces.ICPSkinSettings.columns_class to registry')  # noqa
    record = Record(field.Choice(title=_(u'Content columns related class'),
                                 description=_(u'CSS content columns class that will be applied to body.'),  # noqa
                                 required=False,
                                 values=[
                                     u'content-1',
                                     u'content-2',
                                     u'content-3',
                                     u'content-4',
                                 ],
                                 default=None),
                    value=None)
    records['cpskin.core.interfaces.ICPSkinSettings.columns_class'] = record

    logger.info(
        'Adding cpskin.core.interfaces.ICPSkinSettings.navigation_class to registry')  # noqa
    record = Record(field.Choice(title=_(u'Navigation related class'),
                                 description=_(u'CSS navigation class that will be applied to body.'),  # noqa
                                 required=False,
                                 values=[
                                     u'navigation-1',
                                     u'navigation-2',
                                     u'navigation-3',
                                     u'navigation-4',
                                 ],
                                 default=None),
                    value=None)
    records['cpskin.core.interfaces.ICPSkinSettings.navigation_class'] = record


    logger.info(
        'Adding cpskin.core.interfaces.ICPSkinSettings.footer_class to registry')  # noqa
    record = Record(field.Choice(title=_(u'Footer related class'),
                                 description=_(u'CSS footer class that will be applied to body.'),  # noqa
                                 required=False,
                                 values=[
                                     u'footer-1',
                                     u'footer-2',
                                     u'footer-3',
                                     u'footer-4',
                                 ],
                                 default=None),
                    value=None)
    records['cpskin.core.interfaces.ICPSkinSettings.footer_class'] = record


def addIndexedTaxonomiesToRegistry():
    registry = getUtility(IRegistry)
    records = registry.records

    if 'cpskin.core.interfaces.ICPSkinSettings.indexed_taxonomies' in records:  # noqa
        return

    logger.info(
        'Adding cpskin.core.interfaces.ICPSkinSettings.indexed_taxonomies to registry')  # noqa
    record = Record(field.Text(title=_(u'Taxonomies to index'),
                               description=_(u'List of taxonomy IDs (one per line) that should be indexed.'),
                               required=False,
                               default=u'types_activites\n'),
                    value=u'types_activites\n')
    records['cpskin.core.interfaces.ICPSkinSettings.indexed_taxonomies'] = record


def set_googleapi_key():
    record_name = 'collective.geo.settings.interfaces.IGeoSettings.googleapi'
    default_value = 'AIzaSyDmbfEFrVcZ_x7Snn4Kv_WkqCmiZXn01rY'
    value = api.portal.get_registry_record(record_name)
    if value == default_value:
        api.portal.set_registry_record(
            record_name, 'ABQIAAAAaKes6QWqobpCx2AOamo-shTwM0brOpm')


def upgrade_front_page():
    portal = api.portal.get()
    if not portal.hasObject('front-page'):
        frontPage = api.content.create(
            container=portal,
            type='Document',
            id='front-page',
            title='Bienvenue chez IMIO'
        )
    else:
        frontPage = getattr(portal, 'front-page', None)
    # if frontPage is not None:
    #     frontPage.setExcludeFromNav(True)
    setPageText(portal, frontPage, 'cpskin-frontpage-setup')


def add_default_taxonomies():
    taxonomies = [{
        'filename': '@@taxonomy-settings-categories-evenements',
        'name': 'categoriesevenements',
        'title': u'Catégories événements',
        'field_description': '',
        'default_language': 'fr'
    }, {
        'filename': '@@taxonomy-settings-thematiques',
        'name': 'thematiques',
        'title': u'Thématiques',
        'field_description': '',
        'default_language': 'fr'
    }, {
        'filename': '@@taxonomy-settings-types-activites',
        'name': 'types_activites',
        'title': u"Types d'activités",
        'field_description': '',
        'default_language': 'fr'
    }, {
        'filename': '@@taxonomy-settings-types-organisations',
        'name': 'types_organisations',
        'title': u"Types d'organisations",
        'field_description': '',
        'default_language': 'fr'
    }]
    for tax in taxonomies:
        # TODO check if no taxonomy exist with same id
        portal = api.portal.get()
        taxonomy = registerTaxonomy(
            portal,
            name=tax['name'],
            title=tax['title'],
            description=tax['field_description'],
            default_language=tax['default_language']
        )

        # Import
        adapter = TaxonomyImportExportAdapter(portal)
        data_path = os.path.join(os.path.dirname(__file__), 'data')
        file_path = os.path.join(data_path, tax['filename'])
        data = open(file_path, 'r').read(),
        import_file = data[0]
        adapter.importDocument(taxonomy, import_file)
        del tax['filename']
        # {'field_description': None, 'title': u'Cat\xe9gorie', 'field_title': u'Cat\xe9gorie', 'description': None, 'is_required': False, 'default_language': u'fr', 'write_permission': None, 'is_single_select': False, 'name': 'collective.taxonomy.generated.categorievent'}
        # {'field_description': '', 'title': 'Cat\xc3\xa9gories \xc3\xa9v\xc3\xa9nements','description': '',  'is_required': False, 'default_language': 'fr', 'write_permission': None, 'is_single_select': False, 'name': 'collective.taxonomy.generated.categoriesevenements'}

        # {'field_description': None,
        # 'title': u'Cat\xe9gorie',
        tax['field_title'] = tax['title']
        # 'description': None,
        tax['is_required'] = False
        tax['write_permission'] = None
        tax['is_single_select'] = False
        # 'name': 'collective.taxonomy.generated.categorievent'}
        taxonomy.registerBehavior(**tax)
        logger.info('Taxonomy {0} imported'.format(tax['name']))



# -*- coding: utf-8 -*-
from collective.dexteritytextindexer.behavior import IDexterityTextIndexer
from cpskin.core.behaviors.indexview import ICpskinIndexViewSettings
from cpskin.core.behaviors.metadata import IAdditionalSearchableText
from cpskin.core.behaviors.metadata import IHiddenTags
from cpskin.core.behaviors.metadata import IRelatedContacts
from cpskin.core.interfaces import ICPSkinCoreLayer
from cpskin.core.testing import CPSKIN_CORE_INTEGRATION_TESTING
from cpskin.core.utils import add_behavior
from cpskin.core.utils import remove_behavior
from plone import api
from plone.app.testing import applyProfile
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.textfield.value import RichTextValue
from zope.interface import alsoProvides

import unittest


class TestBehaviors(unittest.TestCase):

    layer = CPSKIN_CORE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        alsoProvides(self.request, ICPSkinCoreLayer)
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.folder = api.content.create(self.portal, "Folder", "folder")
        self.document = api.content.create(self.folder, "Document", "document")
        self.collection = api.content.create(self.folder, "Collection", "collection")

    def test_use_slider_image_scale(self):
        add_behavior("Collection", ICpskinIndexViewSettings.__identifier__)
        slider_image_scale = getattr(self.collection, "slider_image_scale")
        self.assertEqual(slider_image_scale, "slider")
        remove_behavior("Collection", ICpskinIndexViewSettings.__identifier__)

    def test_use_carousel_image_scale(self):
        add_behavior("Collection", ICpskinIndexViewSettings.__identifier__)
        carousel_image_scale = getattr(self.collection, "carousel_image_scale")
        self.assertEqual(carousel_image_scale, "carousel")

    def test_use_link_text(self):
        add_behavior("Collection", ICpskinIndexViewSettings.__identifier__)
        link_text = getattr(self.collection, "link_text")
        self.assertEqual(link_text, "")

    def test_use_keyword_homepage(self):
        add_behavior("Collection", ICpskinIndexViewSettings.__identifier__)
        index_view_keywords = getattr(self.collection, "index_view_keywords")
        self.assertEqual(
            index_view_keywords, None, "Index view keyword is empty by default"
        )

    def test_related_contacts(self):
        add_behavior("Document", IRelatedContacts.__identifier__)
        aboveContentContact = getattr(self.document, "aboveContentContact")
        self.assertEqual(aboveContentContact, [])
        belowContentContact = getattr(self.document, "belowContentContact")
        self.assertFalse(belowContentContact)
        remove_behavior("Document", IRelatedContacts.__identifier__)

    def test_related_contacts_see_map(self):
        add_behavior("Document", IRelatedContacts.__identifier__)
        see_map = getattr(self.document, "see_map")
        self.assertTrue(see_map)
        setattr(self.document, "see_map", False)
        see_map = getattr(self.document, "see_map")
        self.assertFalse(see_map)
        remove_behavior("Document", IRelatedContacts.__identifier__)

    def test_related_contacts_see_logo_in_popup(self):
        add_behavior("Document", IRelatedContacts.__identifier__)
        see_logo_in_popup = getattr(self.document, "see_logo_in_popup")
        self.assertTrue(see_logo_in_popup)
        setattr(self.document, "see_logo_in_popup", False)
        see_logo_in_popup = getattr(self.document, "see_logo_in_popup")
        self.assertFalse(see_logo_in_popup)
        remove_behavior("Document", IRelatedContacts.__identifier__)

    def test_hidden_tags(self):
        add_behavior("Document", IHiddenTags.__identifier__)
        hiddenTags = getattr(self.document, "hiddenTags")
        self.assertEqual(hiddenTags, None)
        self.document.hiddenTags = ("mon-test",)
        self.document.reindexObject()
        hiddenTags = getattr(self.document, "hiddenTags")
        self.assertEqual(hiddenTags, ("mon-test",))
        catalog = api.portal.get_tool("portal_catalog")
        query = {"hiddenTags": "mon-test"}
        brains = catalog(query)
        self.assertEqual(len(brains), 1)
        self.assertEqual(brains[0].getObject(), self.document)

    def test_taxonomy_category(self):
        applyProfile(self.portal, "collective.taxonomy:default")
        add_behavior("Collection", ICpskinIndexViewSettings.__identifier__)
        test_taxonomy_category = getattr(self.collection, "taxonomy_category")
        self.assertEqual(test_taxonomy_category, "")

    def test_indexview_hide_title(self):
        add_behavior("Collection", ICpskinIndexViewSettings.__identifier__)
        hide_title = getattr(self.collection, "hide_title")
        self.assertEqual(hide_title, False)

    def test_additional_searchable_text(self):
        catalog = api.portal.get_tool("portal_catalog")
        add_behavior("Document", IAdditionalSearchableText.__identifier__)
        add_behavior("Document", IDexterityTextIndexer.__identifier__)
        additional_searchable_text = getattr(
            self.document, "additional_searchable_text"
        )
        self.assertEqual(additional_searchable_text, None)
        additional_searchable_text = setattr(
            self.document, "additional_searchable_text", "trash"
        )
        additional_searchable_text = getattr(
            self.document, "additional_searchable_text"
        )
        self.assertEqual(additional_searchable_text, "trash")

        # check that additional_searchable_text is correctly indexed
        # and that body text is still indexed
        richtextvalue = RichTextValue(
            u'richtext_value',
            'text/plain',
            'text/html'
        )
        self.document.text = richtextvalue
        self.document.reindexObject()

        brain = catalog(UID=self.document.UID())[0]
        indexes = catalog.getIndexDataForRID(brain.getRID())
        searchable_text = indexes['SearchableText']
        self.assertTrue("trash" in searchable_text)
        self.assertTrue("document" in searchable_text)
        self.assertTrue("richtext_value" in searchable_text)



# -*- coding: utf-8 -*-
from cpskin.locales import CPSkinMessageFactory as _
from plone.autoform import directives
from plone.autoform.interfaces import IFormFieldProvider
from plone.supermodel import model
from zope import schema
from zope.interface import provider


@provider(IFormFieldProvider)
class IBooking(model.Schema):

    directives.order_after(booking_type='IDublinCore.description')
    booking_type = schema.Choice(
        title=_(u'Booking type'),
        required=False,
        vocabulary=u'cpskin.core.vocabularies.booking_types',
    )

    directives.order_after(booking_price='.booking_type')
    booking_price = schema.Text(
        title=_(u'Price'),
        required=False,
    )

    directives.order_after(booking_url='.booking_price')
    booking_url = schema.URI(
        title=_(u'Booking URL'),
        required=False,
    )



# -*- coding: utf-8 -*-
from setuptools import find_packages
from setuptools import setup

version = '0.13.45'

long_description = (
    open('README.rst').read()
    + '\n' +
    'Contributors\n'
    '============\n'
    + '\n' +
    open('CONTRIBUTORS.rst').read()
    + '\n' +
    open('CHANGES.rst').read()
    + '\n')

setup(name='cpskin.core',
      version=version,
      description='Core package for cpskin',
      long_description=long_description,
      classifiers=[
          "Environment :: Web Environment",
          "Programming Language :: Python",
          "Programming Language :: Python :: 2.7",
          "Framework :: Plone",
          "Framework :: Plone :: 4.2",
          "Framework :: Plone :: 4.3",
      ],
      keywords='',
      author='IMIO',
      author_email='support@imio.be',
      url='https://github.com/imio/cpskin.core',
      license='gpl',
      packages=find_packages(),
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'cpskin.locales',
          'cpskin.citizen',
          'cpskin.minisite',
          'cpskin.menu',
          'plone.api',
          'plone.app.contenttypes',
          'Plone',
          'archetypes.schemaextender',
          'imio.ckeditortemplates',
          'collective.directory',
          'collective.monkeypatcher',
          'imio.media',
          'collective.plonetruegallery',
          'collective.geo.faceted',  # include collective.geo.leaflet
          'plone.behavior',
          'collective.sticky',
          'collective.quickupload',
          'wildcard.foldercontents',
          'eea.facetednavigation',
          'collective.iconifieddocumentactions',
          'collective.z3cform.keywordwidget',
          'httpagentparser',
          'collective.contact.core',
          'collective.folderishtypes',
          'collective.taxonomy',
          'collective.navigationtoggle',
          'imio.dashboard',
          'collective.jsonify',
          'phonenumbers',
          'collective.contact.facetednav',
          'collective.dexteritytextindexer',
          'plone.app.imagecropping',
          'plone.app.multilingual',
          'plone.app.event',
          'z3c.jbot',
          'sc.social.like',
          'collective.js.fancybox',
          'imio.gdpr',
          'collective.printrss',
          'collective.recaptcha',
          'collective.sendinblue<2.0',
          'collective.lesscss',
          'collective.anysurfer',
      ],
      extras_require={
          'test': [
              'plone.app.robotframework',
              'plone.app.multilingual',
              'Products.contentmigration',
              'ipdb',
              'cpskin.workflow'
          ]
      },
      entry_points={},
      )



from plone import api
from zope.component import getMultiAdapter

from plonetheme.sunburst.browser.sunburstview import SunburstView as BaseSunburstView


class SunburstView(BaseSunburstView):

    def getColumnsClasses(self, view=None):
        """Determine whether a column should be shown. The left column is
        called plone.leftcolumn; the right column is called plone.rightcolumn.
        CPSkin : this is overriden to be able to take right portlets into
        special right actions panel (#17748).
        """
        portal_registry = api.portal.get_tool('portal_registry')
        enabled = portal_registry['cpskin.core.interfaces.ICPSkinSettings.show_portlets_in_right_actions_panel']  # noqa
        if not enabled:
            # CPSkin : if show_portlets_in_right_actions_panel is not enabled,
            # we use default columns calculation
            return super(SunburstView, self).getColumnsClasses(view)

        plone_view = getMultiAdapter(
            (self.context, self.request), name=u'plone')
        portal_state = getMultiAdapter(
            (self.context, self.request), name=u'plone_portal_state')

        sl = plone_view.have_portlets('plone.leftcolumn', view=view)
        # CPSkin : if show_portlets_in_right_actions_panel is enabled, we
        # don't want any right portlets consideration
        sr = False

        isRTL = portal_state.is_rtl()

        # pre-fill dictionary
        columns = dict(one="", content="", two="")

        if not sl and not sr:
            # we don't have columns, thus conten takes the whole width
            columns['content'] = "width-full position-0"

        elif sl and sr:
            # In case we have both columns, content takes 50% of the whole
            # width and the rest 50% is spread between the columns
            columns['one'] = "width-1:4 position-0"
            columns['content'] = "width-1:2 position-1:4"
            columns['two'] = "width-1:4 position-3:4"

        elif (sr and not sl) and isRTL:
            # We have right column and we are in RTL language
            columns['content'] = "width-3:4 position-1:4"
            columns['two'] = "width-1:4 position-0"

        elif (sr and not sl) and not isRTL:
            # We have right column and we are NOT in RTL language
            columns['content'] = "width-3:4 position-0"
            columns['two'] = "width-1:4 position-3:4"

        elif (sl and not sr) and isRTL:
            # We have left column and we are in RTL language
            columns['one'] = "width-1:4 position-3:4"
            columns['content'] = "width-3:4 position-0"

        elif (sl and not sr) and not isRTL:
            # We have left column and we are in NOT RTL language
            columns['one'] = "width-1:4 position-0"
            columns['content'] = "width-3:4 position-1:4"

        # append cell to each css-string
        for key, value in columns.items():
            columns[key] = "cell " + value

        return columns



# -*- coding: utf-8 -*-
from collective.geo.behaviour.interfaces import ICoordinates
from cpskin.core.behaviors.metadata import IHiddenTags
from cpskin.core.testing import CPSKIN_CORE_INTEGRATION_TESTING
from cpskin.core.utils import add_behavior
from cpskin.core.utils import remove_behavior
from cpskin.core.utils import set_exclude_from_nav
from plone import api
from plone.app.testing import applyProfile
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from zope.component import getUtility
from zope.schema.interfaces import IVocabularyFactory

import unittest


class TestVocabularies(unittest.TestCase):

    layer = CPSKIN_CORE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']

    def test_contact_fields_vocabulary(self):
        applyProfile(self.portal, 'collective.contact.core:default')
        name = 'cpskin.core.vocabularies.contact_fields'
        factory = getUtility(IVocabularyFactory, name)
        vocabulary = factory(self.portal)
        keys = vocabulary.by_value.keys()
        self.assertNotIn('im_handle', keys)
        self.assertIn('street', keys)
        self.assertIn('number', keys)
        self.assertIn('zip_code', keys)
        self.assertIn('city', keys)
        self.assertIn('activity', keys)
        self.assertIn('title', keys)
        titles = [val.title for val in vocabulary.by_value.values()]
        self.assertIn(u'Contact details: Email', titles)

    def test_hidden_tags_vocabulary(self):
        name = 'cpskin.core.vocabularies.hiddenTags'
        factory = getUtility(IVocabularyFactory, name)
        vocabulary = factory(self.portal)
        keys = vocabulary.by_value.keys()
        self.assertEqual(keys, [u'a-la-une', u'homepage'])
        add_behavior('Document', IHiddenTags.__identifier__)
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        api.content.create(container=self.portal,
                           type='Document', title='document')
        document = self.portal.document
        document.hiddenTags = [u'mot clé caché']
        document.reindexObject()
        vocabulary = factory(self.portal)
        keys = vocabulary.by_value.keys()
        self.assertEqual(
            keys, [u'mot cl\xe9 cach\xe9', u'a-la-une', u'homepage'])

    def test_geo_types_vocabulary(self):
        name = 'cpskin.core.vocabularies.geo_types'
        factory = getUtility(IVocabularyFactory, name)
        vocabulary = factory(self.portal)
        keys = vocabulary.by_value.keys()
        self.assertEqual(len(keys), 0)
        add_behavior('Document', ICoordinates.__identifier__)

        vocabulary = factory(self.portal)
        keys = vocabulary.by_value.keys()
        self.assertEqual(len(keys), 1)
        self.assertIn(u'Document', keys)

        remove_behavior('Document', ICoordinates.__identifier__)

    def test_action_menu_eligible_vocabulary(self):
        name = 'cpskin.core.vocabularies.action_menu_eligible'
        factory = getUtility(IVocabularyFactory, name)
        vocabulary = factory(self.portal)
        orig_keys = vocabulary.by_value.keys()
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        testfolder = api.content.create(container=self.portal,
                                        type='Folder', title='testfolder')
        api.content.transition(obj=testfolder, transition='publish')
        vocabulary = factory(self.portal)
        keys = vocabulary.by_value.keys()
        self.assertEqual(len(keys), len(orig_keys))
        api.content.create(container=self.portal,
                           type='Document', title='testdocument')
        vocabulary = factory(self.portal)
        keys = vocabulary.by_value.keys()
        self.assertEqual(len(keys), len(orig_keys))
        set_exclude_from_nav(testfolder)
        testfolder.reindexObject()
        vocabulary = factory(self.portal)
        keys = vocabulary.by_value.keys()
        self.assertEqual(len(keys), len(orig_keys) + 1)



# -*- coding: utf-8 -*-
from collective.contact.core import _ as CCMF
from collective.contact.core.behaviors import IContactDetails
from cpskin.core.browser.widget import multiline_field_widget
from cpskin.locales import CPSkinMessageFactory as _
from plone.autoform import directives as form
from plone.autoform.interfaces import IFormFieldProvider
from zope import schema
from zope.interface import provider

import re


_PHONE_RE = re.compile(r'[+]?[0-9 \(\)\/\.]*$')


class InvalidPhone(schema.ValidationError):
    """Exception for invalid address"""
    __doc__ = _(u'Invalid phone')


def validatePhone(value):
    """Simple email validator"""
    if not _PHONE_RE.match(value):
        raise InvalidPhone(value)
    return True


def validate_phones(values):
    for value in values:
        validatePhone(value)
    return True


@provider(IFormFieldProvider)
class IDirectoryContactDetails(IContactDetails):

    form.widget(phone=multiline_field_widget)
    phone = schema.List(
        title=_(u'Phones'),
        required=False,
        constraint=validate_phones,
    )

    form.widget(cell_phone=multiline_field_widget)
    cell_phone = schema.List(
        title=CCMF(u'Cell phone'),
        required=False,
        constraint=validate_phones,
    )



# -*- coding: utf-8 -*-
from cpskin.core.utils import image_scale
from plone import api
from Products.Five.browser import BrowserView


class DescriptionView(BrowserView):

    def extra_output_and_no_folderish(self):
        return self.context.text.output_relative_to(self.context)



# -*- coding: utf-8 -*-
from plone import api
from plone.app.layout.viewlets import common
from plone.registry.interfaces import IRegistry
from Products.CMFCore.WorkflowCore import WorkflowException
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from sc.social.like.browser.viewlets import SocialLikesViewlet as SLV
from zope.component import getUtility

import operator


class SocialViewlet(common.ViewletBase):

    index = ViewPageTemplateFile('social.pt')

    def getSocialLinks(self):
        registry = getUtility(IRegistry)
        links_dict = registry['cpskin.core.socialviewlet']
        links_ordered = sorted(links_dict.itervalues(), key=operator.itemgetter(0))
        # Fix bad encoded cpskin.core.socialviewlet registry :
        links_ordered = [li for li in links_ordered if len(li) == 3]
        return links_ordered


class SocialLikesViewlet(SLV):
    def enabled(self):
        """Check if the viewlet should be visible on this context."""
        allowed_states = api.portal.get_registry_record(
            'cpskin.core.social.allowed_states',
            default=['published_and_shown', 'published_and_hidden'],
        )
        try:
            published = api.content.get_state(self.context) in allowed_states
        except WorkflowException:
            # no workflow on context, like in site root
            published = True
        return all([published, self.helper.enabled(), self.plugins()])



# -*- coding: utf-8 -*-
from Acquisition import aq_base
from collective.contact.core.browser.address import get_address
from collective.contact.core.interfaces import IContactable
from collective.geo.json.browser.jsonview import get_marker_image
from collective.geo.leaflet.interfaces import IGeoMap
from collective.geo.mapwidget import utils
from cpskin.core.utils import format_phone
from plone import api
from plone.app.layout.viewlets import common
from plone.dexterity.utils import safe_unicode
from plone.dexterity.utils import safe_utf8
from plone.outputfilters.filters.resolveuid_and_caption import (
    ResolveUIDAndCaptionFilter,
)  # noqa
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from pygeoif.geometry import as_shape

import geojson
import logging
import Missing


logger = logging.getLogger("cpskin.core related contacts viewlet")


class RelatedContactsViewlet(common.ViewletBase):

    index = ViewPageTemplateFile("related_contacts.pt")
    address_fields = (
        "street",
        "number",
        "zip_code",
        "city",
        "additional_address_details",
        "region",
        "country",
    )
    coordinates_fields = (
        "phone",
        "cell_phone",
        "fax",
        "email",
        "im_handle",
        "website",
        "schedule",
    )
    ignore_fields = ("title",)

    def __init__(
        self, context, request, view, manager=None, field="", selected=""
    ):  # noqa
        super(RelatedContactsViewlet, self).__init__(context, request, view, manager)
        self.field = field
        self.selected = selected
        self.pc = api.portal.get_tool("portal_catalog")

    def available(self):
        context = aq_base(self.context)
        return bool(getattr(context, self.field, None))

    @property
    def selected_fields(self):
        return getattr(self.context, self.selected, None)

    def get_contacts(self):
        contacts = []
        if isinstance(self.field, list):
            related_contacts = []
            for field in self.field:
                related_contacts += getattr(self.context, field, [])
        else:
            related_contacts = getattr(self.context, self.field, [])
        for related_contact in related_contacts:
            if related_contact.isBroken():
                related_contacts.remove(related_contact)
                setattr(self.context, self.field, related_contacts)
            obj = related_contact.to_object
            if obj not in contacts:
                contacts.append(obj)
        return contacts

    def get_title(self, contact):
        if self.in_fields("title"):
            return u'<span class="related-contact-title">{0}</span>'.format(
                contact.title
            )
        else:
            return False

    def in_fields(self, field):
        return field in self.selected_fields

    def get_field(self, contact, field_name):
        if field_name is "id":
            return getattr(contact, field_name, "")
        if field_name not in self.selected_fields:
            return ""
        if field_name in self.address_fields:
            contactable = IContactable(contact)
            details = contactable.get_contact_details()
            return details["address"].get(field_name)
        # field = getattr(contact, field_name, '')
        # find way to check if field is richetext or image or simple field
        if getattr(getattr(contact, field_name, ""), "raw", None):
            if getattr(contact, field_name, ""):
                text = getattr(contact, field_name).raw
                text = text.replace("http://resolveuid/", "resolveuid/")
                parser = ResolveUIDAndCaptionFilter(contact)
                transform_text = parser(text)
                return transform_text if transform_text else ""
        if field_name in ["logo", "photo"]:
            if getattr(contact, field_name, ""):
                img = contact.unrestrictedTraverse("@@images")
                logo = img.scale(field_name)
                return logo.tag() if logo.tag() else ""
        if field_name == "schedule":
            from plone.directives import dexterity

            display = dexterity.DisplayForm(contact, self.request)
            display.update()
            if display.w.get("IScheduledContent.schedule", None):
                return display.w.get("IScheduledContent.schedule").render()
            else:
                return ""
        if field_name in ["phone", "cell_phone", "fax"]:
            phones = getattr(contact, field_name, "")
            if not phones:
                return False
            if not isinstance(phones, list):
                phones = [getattr(contact, field_name)]
            return [format_phone(phone) for phone in phones]
        if field_name in ["position"]:
            positions = [pos.title.strip() for pos in contact.get_held_positions()]
            return ", ".join(positions)
        return getattr(contact, field_name, "")

    def has_address(self):
        i = 0
        for address_field in self.address_fields:
            if address_field in self.selected_fields:
                i += 1
        return i >= 3

    def fields_without_address(self):
        fields = []
        for selected_field in self.selected_fields:
            if (
                selected_field not in self.address_fields
                and selected_field not in self.ignore_fields
                and selected_field not in self.coordinates_fields
            ):
                fields.append(selected_field)
        return fields

    def get_website(self, contact):
        website = self.get_field(contact, "website")
        if website.startswith("http"):
            url = website
            website_name = website.replace("http://", "")
        elif website.startswith("https"):
            url = website
            website_name = website.replace("https://", "")
        else:
            url = "http://{0}".format(website)
            website_name = website
        html = ""
        html += '<a class="website" href="{0}" target="_blank">{1}</a>'.format(
            url, website_name
        )
        return html

    def see_map_link(self, contact):
        if self.available:
            obj = contact
            if contact.use_parent_address:
                obj = contact.aq_parent
            brain = self.pc.unrestrictedSearchResults(UID=obj.UID())[0]
            if brain.zgeo_geometry == Missing.Value:
                return False
            if not getattr(self.context, "see_map", True):
                return False
            return True
        return False


class AboveRelatedContactsViewlet(RelatedContactsViewlet):
    def __init__(self, context, request, view, manager=None):
        field = "aboveContentContact"
        selected = "aboveVisbileFields"
        super(AboveRelatedContactsViewlet, self).__init__(
            context, request, view, manager, field, selected
        )


class BelowRelatedContactsViewlet(RelatedContactsViewlet):
    def __init__(self, context, request, view, manager=None):
        field = "belowContentContact"
        selected = "belowVisbileFields"
        super(BelowRelatedContactsViewlet, self).__init__(
            context, request, view, manager, field, selected
        )

    def get_title(self, contact):
        if self.in_fields("title"):
            return u'<a href="{0}" target="_blank"><h4>{1}</h4></a>'.format(
                contact.absolute_url(), safe_unicode(contact.title)
            )
        else:
            return False


class RelatedContactsMapViewlet(RelatedContactsViewlet):
    index = ViewPageTemplateFile("related_contacts_map.pt")

    def __init__(self, context, request, view, manager=None):
        self.fields = ["aboveContentContact", "belowContentContact"]
        super(RelatedContactsMapViewlet, self).__init__(
            context, request, view, manager, self.fields
        )

    def available(self):
        context = aq_base(self.context)
        see_map = getattr(context, "see_map", False)
        empty_content = True
        for field in self.fields:
            if len(getattr(context, field, [])) > 0:
                empty_content = False
        return see_map and not empty_content

    @property
    def geomap(self):
        return IGeoMap(self.context)

    def data_geojson(self):
        style = {}
        global_style = utils.get_feature_styles(self.context)
        style["fill"] = global_style["polygoncolor"]
        style["stroke"] = global_style["linecolor"]
        style["width"] = global_style["linewidth"]
        if global_style.get("marker_image", None):
            img = get_marker_image(self.context, global_style["marker_image"])
            style["image"] = img
        else:
            style["image"] = None
        json_result = []
        self.pc = api.portal.get_tool("portal_catalog")
        for contact in self.get_contacts():
            brain = self.pc.unrestrictedSearchResults(UID=contact.UID())[0]
            if contact.use_parent_address:
                brain = self.pc.unrestrictedSearchResults(UID=contact.aq_parent.UID())[
                    0
                ]
            if brain.zgeo_geometry == Missing.Value:
                continue
            if brain.collective_geo_styles == Missing.Value:
                continue
            if brain.collective_geo_styles.get(
                "use_custom_styles", False
            ) and brain.collective_geo_styles.get("marker_image", None):
                img = get_marker_image(
                    self.context, brain.collective_geo_styles["marker_image"]
                )
                style["image"] = img
            geom = {
                "type": brain.zgeo_geometry["type"],
                "coordinates": brain.zgeo_geometry["coordinates"],
            }
            if geom["coordinates"]:
                if geom["type"]:
                    classes = geom["type"].lower() + " "
                else:
                    classes = ""
                address = get_address(contact)
                number = ""
                if address.get("number", None):
                    number = ", {0}".format(address["number"])
                formated_address = "{0}{1}<br />{2} {3}".format(
                    safe_utf8(address.get("street") or ""),
                    number,
                    address.get("zip_code") or "",
                    safe_utf8(address.get("city") or ""),
                )
                img = ""
                if self.context.see_logo_in_popup:
                    acc = getattr(contact, "logo", None)
                    if acc and acc.filename:
                        img = "{0}/@@images/logo/thumb".format(contact.absolute_url())
                classes += brain.getPath().split("/")[-2].replace(".", "-")
                json_result.append(
                    geojson.Feature(
                        id=contact.id.replace(".", "-"),
                        geometry=as_shape(geom),
                        style=style,
                        properties={
                            "title": brain.Title,
                            "description": brain.Description,
                            "style": style,
                            "url": brain.getURL(),
                            "classes": classes,
                            "image": img,
                            "address": formated_address,
                        },
                    )
                )
        feature_collection = geojson.FeatureCollection(json_result)
        feature_collection.update({"title": self.context.title})
        return geojson.dumps(feature_collection)



# -*- coding: utf-8 -*-

from cpskin.minisite.interfaces import IInPortal
from plone import api
from plone.app.layout.viewlets import common


class TopCollapseViewlet(common.ViewletBase):

    def available(self):
        if not IInPortal.providedBy(self.request):
            return False
        return api.portal.get_registry_record(
            'cpskin.core.interfaces.ICPSkinSettings.collapse_minisite_menu',
            default=False,
        )



# -*- coding: utf-8 -*-

from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from collective.contact.core.content.organization import IOrganization
from collective.folderishtypes.interfaces import IFolderishType
from plone import api
from plone.app.layout.viewlets import common


class OrganizationGalleryViewlet(common.ViewletBase):

    index = ViewPageTemplateFile('organization_gallery.pt')

    def available(self):
        """
        Condition on the content types wanted for the gallery viewlet
        (it was first meant only for Organizations), now it is also used for
        folderish content types (collective.folderish)
        """
        context = self.context
        return IOrganization.providedBy(context) or IFolderishType.providedBy(context)

    def get_photos(self):
        brains = api.content.find(
            context=self.context,
            depth=1,
            portal_type='Image'
        )
        return [b.getObject() for b in brains]

    def image_url(self, obj, default_scale='preview'):
        url = ''
        images = obj.restrictedTraverse('@@images')
        image = images.scale('image', scale=default_scale)
        if image:
            url = image.url
        return url



# -*- coding: utf-8 -*-

from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from cpskin.minisite.interfaces import IInMinisiteBase
from cpskin.minisite.utils import get_minisite_object
from plone import api
from plone.app.layout.viewlets.common import ViewletBase


class CPSkinFooterMinisiteViewlet(ViewletBase):
    render = ViewPageTemplateFile('footer_minisite.pt')

    def available(self):
        request = self.request
        if not IInMinisiteBase.providedBy(request):
            return False
        content = self.content()
        if content is None:
            return False
        elif api.content.get_state(obj=content) != 'published_and_hidden':
            return False
        return True

    def content(self):
        request = self.request
        minisite_obj = get_minisite_object(request)
        document = getattr(minisite_obj, 'footer-mini-site', None)
        return document



# -*- coding: utf-8 -*-
from collective.dexteritytextindexer.behavior import IDexterityTextIndexer
from cpskin.core.behaviors.metadata import IAdditionalSearchableText
from cpskin.core.interfaces import ICPSkinCoreLayer
from cpskin.core.testing import CPSKIN_CORE_INTEGRATION_TESTING
from cpskin.core.utils import add_behavior
from plone import api
from plone.app.testing import TEST_USER_ID
from plone.app.testing import applyProfile
from plone.app.testing import setRoles
from plone.app.textfield.value import RichTextValue
from zope.interface import alsoProvides

import unittest


class TestOrganizations(unittest.TestCase):

    layer = CPSKIN_CORE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        alsoProvides(self.request, ICPSkinCoreLayer)
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        portal_languages = api.portal.get_tool("portal_languages")
        portal_languages.addSupportedLanguage("fr")
        applyProfile(self.portal, "collective.contact.core:default")
        add_behavior("organization", "collective.taxonomy.generated.types_activites")
        add_behavior("organization", IAdditionalSearchableText.__identifier__)
        add_behavior("organization", IDexterityTextIndexer.__identifier__)
        self.directory = api.content.create(self.portal, "directory", "directory")
        self.organization = api.content.create(self.directory, "organization", "orga")
        self.organization.language = 'fr'

    def test_organization(self):
        catalog = api.portal.get_tool("portal_catalog")
        self.organization.additional_searchable_text = "additional"
        richtextvalue = RichTextValue(u"activity_value", "text/plain", "text/html")
        self.organization.activity = richtextvalue
        richtextvalue = RichTextValue(
            u"informations_complementaires_value", "text/plain", "text/html"
        )
        self.organization.informations_complementaires = richtextvalue
        self.organization.taxonomy_typesactivites = [u"nvf7kib5ed", u"f8bvklelyb"]
        self.organization.reindexObject()

        brain = catalog(UID=self.organization.UID())[0]
        indexes = catalog.getIndexDataForRID(brain.getRID())
        searchable_text = indexes["SearchableText"]
        self.assertTrue("additional" in searchable_text)
        self.assertTrue("activity_value" in searchable_text)
        self.assertTrue("informations_complementaires_value" in searchable_text)
        self.assertTrue("autres" in searchable_text)
        self.assertTrue("animations" in searchable_text)
